import pytest

from environs import EnvError
from environs_serviceurl import service_url

basic_cases = pytest.mark.parametrize(('url', 'expected'), [
    # Basic null scenario
    ('', {
        'host': None,
        'port': None,
        'user': None,
        'password': None,
        'extras': None,
    }),
    # Simple URL, no params, missing fields.
    ('http://example.com', {
        'host': 'example.com',
        'port': None,
        'user': None,
        'password': None,
        'extras': None,
    }),
    #  All the possibile url fields, plus path-extra and query params.
    ('tcp://a_user:a_pass@example.com:1234/silly?spam=eggs&parrots=pining', {
        'host': 'example.com',
        'port': 1234,
        'user': 'a_user',
        'password': 'a_pass',
        'extras': 'silly',
        'spam': 'eggs',
        'parrots': 'pining',
    }),
    # Multiple params with same key should return last key's value.
    ('http://example.com?spam=eggs&spam=spam', {
        'host': 'example.com',
        'port': None,
        'user': None,
        'password': None,
        'extras': None,
        'spam': 'spam',
    }),
    # URL encoded values work
    ('http://:passwith%23@example.com?dir=%2f', {
        'host': 'example.com',
        'port': None,
        'user': None,
        'password': 'passwith#',
        'extras': None,
        'dir': '/',
    }),
    # Throws an error on an unparsable thing.
    ([1], EnvError),
])

default_cases = pytest.mark.parametrize(('url', 'defaults', 'expected'), [
    # Empty default doesn't break.
    ('', {}, {
        'host': None,
        'port': None,
        'user': None,
        'password': None,
        'extras': None,
    }),
    # Simple default appears if url doesn't have that fied.
    ('', {'host': 'localhost'}, {
        'host': 'localhost',
        'port': None,
        'user': None,
        'password': None,
        'extras': None,
    }),
    # Default doesn't appear if url has field.
    ('http://example.com', {'host': 'localhost'}, {
        'host': 'example.com',
        'port': None,
        'user': None,
        'password': None,
        'extras': None,
    }),
    # Default and non-default fields appear together
    ('http://example.com', {'port': 80}, {
        'host': 'example.com',
        'port': 80,
        'user': None,
        'password': None,
        'extras': None,
    }),
    # Defaults for extras/query params work
    ('http://example.com', {'spam': 'eggs'}, {
        'host': 'example.com',
        'port': None,
        'user': None,
        'password': None,
        'extras': None,
        'spam': 'eggs',
    }),
    # Param defaults are overridden.
    ('http://example.com?spam=lovelyspam', {'spam': 'eggs'}, {
        'host': 'example.com',
        'port': None,
        'user': None,
        'password': None,
        'extras': None,
        'spam': 'lovelyspam',
    }),
    # Defaults don't prevent errors on unparseable values.
    ([1], {'host': 'localhost'}, EnvError),
])

extras_cases = pytest.mark.parametrize(('url', 'extras', 'expected'), [
    # When we have an extras name, we get that name in the return, not 'extras'
    ('http://example.com/gandalf', 'wizard', {
        'host': 'example.com',
        'port': None,
        'user': None,
        'password': None,
        'wizard': 'gandalf',
    }),
])


@basic_cases
def test_basics(url, expected):
    if type(expected) == type(type) and issubclass(expected, Exception):
        with pytest.raises(expected):
            service_url(url) == expected
    else:
        assert service_url(url) == expected


@default_cases
def test_defaults(url, defaults, expected):
    if type(expected) == type(type) and issubclass(expected, Exception):
        with pytest.raises(expected):
            service_url(url, defaults=defaults) == expected
    else:
        assert service_url(url, defaults=defaults) == expected


@extras_cases
def test_extras_name(url, extras, expected):
    if type(expected) == type(type) and issubclass(expected, Exception):
        with pytest.raises(expected):
            service_url(url, extras_name=extras) == expected
    else:
        assert service_url(url, extras_name=extras) == expected



from setuptools import setup, find_packages


requires = {
    'setup': [
    ],
    'install': [
        'environs',
    ],
    'tests': [
        'pytest',
        'pytest-cov',
        'pytest-flake8',
    ],
}

requires['all'] = list({dep for deps in requires.values() for dep in deps})


def readme():
    with open('README.rst', 'r') as readme_file:
        return readme_file.read()

setup(
    name='environs-serviceurl',
    version='1.0.1',
    description='Add service URL parsing support for environs',
    long_description=readme(),
    url='http://github.com/flowroute/environs-serviceurl',
    author='Dan Root',
    author_email='rootdan+pypi@gmail.com',
    license='MIT',

    packages=find_packages(exclude=['test']),
    setup_requires=requires['setup'],
    install_requires=requires['install'],
    tests_require=requires['tests'],
    extras_require=requires,
    include_package_data=True,
    zip_safe=False,
    platforms='any',

    keywords=[],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)



try:
    from urllib.parse import urlparse, unquote, parse_qs
except ImportError:
    from urlparse import urlparse, parse_qs
    from urllib import unquote

from environs import EnvError


def service_url(url, defaults={}, extras_name='extras'):
    """
    environs handler for URLS, turning a url string into a dictionary
    of its component parts.
    """
    try:
        parsed = urlparse(url)
    except Exception:
        raise EnvError(
            'Service URLS look like: servtype://user:pass@host:port/' +
            extras_name + '?param1=Foo&param2=Bar')

    conf = {
        'host': unquote(parsed.hostname) if parsed.hostname else None,
        'port': parsed.port,
        'user': unquote(parsed.username) if parsed.username else None,
        'password': unquote(parsed.password) if parsed.password else None,
        extras_name: unquote(parsed.path)[1:] if parsed.path else None,
    }

    # If we get multiple values for a given key, use the last.
    extra_params = {k: v[-1] for k, v in parse_qs(parsed.query).items()}
    conf.update(extra_params)

    missing_defaults = {k: v for k, v in defaults.items()
                        if k not in conf or conf[k] is None}
    conf.update(missing_defaults)

    return conf
